  
; ISM pots for emission, reddening, extinction, Srrkowski,
;              and submmPol
; if keyword_set  ps then plot to ps file ps_subpol.ps
; if keyword_set  scale350 and not kerywoprd_set ps then return
;                  afetr scaling factor without plot
; Output:
;  
;  scale350 [g-dust/H-atom] := scale Model [erg/s/sr/g-dust] to Planck 350mu
;              [erg/s/sr/H-atom] to
;
; .r pl_subPol
; .r pl_FIRdef.pro
; .r dirbe_conv.pro
;  
; dirbe_conv, w, flux, ws, dwsm, dwsp, S, xFv, wdirbe, Sdirbe, vF, xvF
; --------
; ============================
pro dirbe_conv, w, flux, ws, dwsm, dwsp, S, xFv, $
                wdirbe, Sdirbe, vF, xvF, pr=pr

; Input:  w, flux [units as in jsm.out]
; Keyword pr if set prints output  
; output: 
;    ws,dws, S := Dirbe data [10^-26 erg/s/sr/H]
;    dirbe, Sdirbe := Model in DIRBE bands
;    central wavelength and nu*Fnu in DIRBE bands
;    vF=   Model scale to 140mic Band
;   xvF= factor to scale jsm "v*F" to S(140)
; -----------------------------------

clicht = 2.9979250d10
w    = w    *1d0
wcm  = w    *1d-4
flux = flux *1d0
fr   = clicht/wcm

mm            = n_elements(fr)
dfr           = fr*0d0
for k         = 2, mm-1 do begin
  dfr(k-1)    = 5d-1 * (fr(k) - fr(k-2))
endfor
  dfr(0)      = (fr(1) - fr(0))
  dfr(mm-1)   = (fr(mm-1) - fr(mm-2))


; ISRF DIBRE data ( band 1,2 are ignored)
; ws = center wavelength, wsp =upper, wsm=lower wavelength (FWHM)
; solor neighborhood fluxes in 10^-26 erg/s/sr/H
ws  = [3.5,  4.9,  12.,   25.,  60.,   100., 140.,  240.]; in microns
dwsm = [3.07, 4.55, 8.6,   16.6, 42.41, 85.0, 133.0, 196.4]
dwsp = [4.01, 5.23, 16.67, 25.4, 70.5,  116., 167.5, 297.]
S   = [0.97, 1.11, 7.16,  3.57, 5.30, 18.6,  22.5,  10.1]
S   = S * 1d-26
; read DIRBE spectral response function:
readcol, '~/IDL/ISRF/dirbe_specrespon.asc', wd, d,d,$
         r3,r5,r12,r25,r60,r100,r140,r240, skipl=101, /sil




wd   = wd *1d0
wa = [w, wd]
wa = reverse(wa(uniq(wa, sort(wa))))

fra   = clicht/wa *1d4
flua  = interpol(flux, w, wa)
r3a   = interpol(r3, wd, wa)
r5a   = interpol(r5, wd, wa)
r12a  = interpol(r12, wd, wa)
r25a  = interpol(r25, wd, wa)
r60a  = interpol(r60, wd, wa)
r100a = interpol(r100, wd, wa)
r140a = interpol(r140, wd, wa)
r240a = interpol(r240, wd, wa)

mma            = n_elements(fra)
dfra           = fra*0d0
for k          = 2, mma-1 do begin
  dfra(k-1)    = 5d-1 * (fra(k) - fra(k-2))
endfor
  dfra(0)      = (fra(1) - fra(0))
  dfra(mma-1)  = (fra(mma-1) - fra(mma-2))

sum = 0
for k         = 0, mm-1 do begin
 if (w(k) ge 1. and w(k) le 4.) then sum = sum + flux(k) * dfr(k)
 endfor


sv3    = 0
srf3   = 0
sr3    = 0
sv5    = 0
srf5   = 0
sr5    = 0
sv12   = 0
srf12  = 0
sr12   = 0
sv25   = 0
srf25  = 0
sr25   = 0
sv60   = 0
srf60  = 0
sr60   = 0
sv100  = 0
srf100 = 0
sr100  = 0
sv140  = 0
srf140 = 0
sr140  = 0
sv240  = 0
srf240 = 0
sr240  = 0


for k   = 0, mma-1 do begin
 srf3   = srf3   + r3a(k)   * flua(k) * dfra(k)
 sr3    = sr3    + r3a(k)   * dfra(k)
 sv3    = sv3    + r3a(k)   * fra(k)  * dfra(k)
 srf5   = srf5   + r5a(k)   * flua(k) * dfra(k)
 sr5    = sr5    + r5a(k)   * dfra(k)
 sv5    = sv5    + r5a(k)   * fra(k)  * dfra(k)
 srf12  = srf12  + r12a(k)  * flua(k) * dfra(k)
 sr12   = sr12   + r12a(k)  * dfra(k)
 sv12   = sv12   + r12a(k)  * fra(k)  * dfra(k)
 srf25  = srf25  + r25a(k)  * flua(k) * dfra(k)
 sr25   = sr25   + r25a(k)  * dfra(k)
 sv25   = sv25   + r25a(k)  * fra(k)  * dfra(k)
 srf60  = srf60  + r60a(k)  * flua(k) * dfra(k)
 sr60   = sr60   + r60a(k)  * dfra(k)
 sv60   = sv60   + r60a(k)  * fra(k)  * dfra(k)
 srf100 = srf100 + r100a(k) * flua(k) * dfra(k)
 sr100  = sr100  + r100a(k) * dfra(k)
 sv100  = sv100  + r100a(k) * fra(k)  * dfra(k)
 srf140 = srf140 + r140a(k) * flua(k) * dfra(k)
 sr140  = sr140  + r140a(k) * dfra(k)
 sv140  = sv140  + r140a(k) * fra(k)  * dfra(k)
 srf240 = srf240 + r240a(k) * flua(k) * dfra(k)
 sr240  = sr240  + r240a(k) * dfra(k)
 sv240  = sv240  + r240a(k) * fra(k)  * dfra(k)

endfor

frdirbe = [sv3/sr3, sv5/sr5, sv12/sr12, sv25/sr25, sv60/sr60,$
           sv100/sr100, sv140/sr140, sv240/sr240]
fratio  = [srf3/sr3, srf5/sr5, srf12/sr12, srf25/sr25, srf60/sr60,$
           srf100/sr100, srf140/sr140, srf240/sr240]

wdirbe = clicht*1e4/ frdirbe

Sdirbe   = clicht*1e4 * fratio/wdirbe


; scale to Dirbe 140mic 
ii     = (where(w le 140.))(0)
vF     = fr*flux
vF140  = (vF(ii-1)-vF(ii))/(w(ii-1)- w(ii)) * (wdirbe(6)-w(ii)) + vF(ii)


xvF    = S(6)/vF140
vF     = xvF * vF
if keyword_set(pr) then print, '  *** sclae factor to 140mu Band: xvF = ', xvf



Sdirbe = Sdirbe * S(6)/Sdirbe(6)

if keyword_set(pr) then begin
   print,  ' w -Model- S      ws-DATA- S'
   print,  '   [mic.]     [1.d-26 erg/s/sr/H] '
   for i=0, 7 do print, format='(1f5.1, 1e10.2, 2x, f5.1, 1e10.2)',$
                        wdirbe(i),Sdirbe(i), ws(i),  S(i)
endif
 
end


; ==================================
pro pl_subPol, ps =ps, filemipol=filemipol

; plot submm polairsation p/p(850mu) : data and model
; if keyword_set  ps then plot to ps file ps_subpol.ps
;

if not keyword_set(filemipol) then filemipol='./Output/emipol.out'
  
; PLanckXXII+15:
wpp   = [849.3, 1381.5,  2096.5,   2997.9] ;,   3189.3]
pp    = [1.,     0.967,   0.989,   0.888]  ;,   1.066]
epp  = [0.,     0.009,   0.012,   0.028]   ;,   0.141]

; Sharif+19
wa       = [ 250.,  350.,  500.,  850.]
psfar    = [1.00,  1.0,  0.93,  1.01] /1.01
psnear   = [1.02,  1.0,  0.93,  0.99] /0.99
psno     = [1.14,  1.0,  0.96,  0.95] /0.95
epsfar  = [0.06, 0.12, 0.10, 0.12] /1.01 
epsnear = [0.06, 0.12, 0.08, 0.12] /0.99
epsno   = [0.08, 0.12, 0.08, 0.12] /0.95

; Ashton18
pa   = [1.000,  1.052, 0.890,  1.00]
epa = [0.1,    0.113, 0.092,  0.0]

; Gandilo+16:
pgc = [0.97, 1.0, 0.93, 1.04] /1.04
pga = [1.07, 1.0, 0.92, 1.08] /1.08
pgi = [1.02, 1.0, 0.93, 1.07] /1.07

epgc = [0.12, 0.14,  0.05, 0.14] /1.04	
epga = [0.08, 0.16,  0.06, 0.16] /1.08	
epgi = [0.09, 0.15,  0.06, 0.15] /1.07

;
; --------------------- Read Model emis.out
;
; -- Dust model: pol(%) normalized to p(850mu).
; syntax alignment radius in units 0.1mic and max radius in units mic.



readcol, filemipol, skipline=2, w, f, fpt, fpac, fpSi, fDark, /sil
wp_submm = w(10:40)
pm    = 100*fpt(10:40)/ f(10:40)
pac   = 100*fpac(10:40) /f(10:40)
psi   = 100*fpSi(10:40) /f(10:40)
pdark = 100*fDark(10:40)/f(10:40)

fak   = pm(10)
pm    = pm/fak
pac   = pac/fak
psi   = psi/fak
pdark = pdark/fak



; no porous model:
;readcol, './BestModel/AvgSerkISM_r201_emis.out', skipline=2, w, f1, fac1, fSi1,; fpsi1, fDark1, fvgr, fvsi, fpah, j_v
;pm1  = 100*fpsi1(10:40)/f1(10:40)
;pm1  = pm1 /pm1(10)


; PLot sub Pol:

if keyword_set(ps) then begin
  window, 0, xsi=600, ysiz=500
  !p.font=0
  !p.thick=4
  !p.charsize =1.5
  setps, file='pl_subPol.ps'
endif

xmin =  180.
xmax = 3050.

plot_oi, [wa, wpp], [pga, pp], /xsty,  /ysty, xthick=2,ythick=2,  $
         xtitle='!3 !9l !3(!9m!7m)', charsize=1.3, $
         ytitle = '!7p / p!D850!9m!7m!N', $
         xtickname=[' ', ' ', ' ', ' ',' ', ' '], $
         xrange=[xmin, xmax], yrange=[0., 1.4], /nodata

xpoly = [xmin, xmin, xmax, xmax]
ypoly = [0.85, 1.15, 1.15, 0.85]

PolyFill, xpoly,ypoly, Color=FSC_Color('light cyan')


oplot, [xmin,xmax],[1.,1.], color=FSC_Color('gray'), thick=1


oploterror, wa*1.005, pgc, wa*0, epgc, thick=1, color=FSC_Color('orange'), psym=3

oploterror, wa*1.015, pga, wa*0, epgc, thick=1, color=FSC_Color('orange'),psym=3
oploterror, wa*1.01, pgi, wa*0, epgc, thick=1, color=FSC_Color('orange'),psym=3

oploterror, wa*0.995, psno, wa*0, epsno, thick=1, color=FSC_Color('magenta'), psym=3
oploterror, wa*0.990, psfar, wa*0, epsno, thick=1, color=FSC_Color('magenta'), psym=3
oploterror, wa*0.985, psnear, wa*0, epsno, thick=1, color=FSC_Color('magenta'), psym=3


oploterror, wa, pa, wa*0, epa, thick=2, color=FSC_Color('brown'), psym=3  
oploterror, wpp, pp, wpp*0, epp, thick=2, color=FSC_Color('blue'), psym=3 

oplot, wa*1.005, pgc,   psym=plotsym(/circle,scale=1.03, /fill, color=FSC_Color('orange'))
oplot, wa*1.015, pga,   psym=plotsym(/circle,scale=1.03, /fill, color=FSC_Color('orange'))
oplot, wa*1.01, pgi,   psym=plotsym(/circle,scale=1.03, /fill, color=FSC_Color('orange'))

oplot, wa*0.995, psno, psym=plotsym(/circle,scale=1.03, /fill, color=FSC_Color('magenta'))
oplot, wa*0.990, psfar, psym=plotsym(/circle,scale=1.03, /fill, color=FSC_Color('magenta'))
oplot, wa*0.985, psnear, psym=plotsym(/circle,scale=1.03, /fill, color=FSC_Color('magenta'))
oplot, wa,      pa,    psym=plotsym(/circle,scale=1.05, /fill, color=FSC_Color('brown'))
oplot, wpp,     pp,   psym=plotsym(/circle,scale=1.05, /fill, color=FSC_Color('blue'))


oplot, wa*1.005, pgc,   psym=plotsym(/circle,scale=1.03)
oplot, wa*1.015, pga,   psym=plotsym(/circle,scale=1.03)
oplot, wa*1.01, pgi,   psym=plotsym(/circle,scale=1.03)
oplot, wa*0.995, psno, psym=plotsym(/circle,scale=1.03)
oplot, wa*0.990, psfar, psym=plotsym(/circle,scale=1.03)
oplot, wa*0.985, psnear, psym=plotsym(/circle,scale=1.03)
oplot, wa, pa,    psym=plotsym(/circle,scale=1.05)
oplot, wpp, pp,   psym=plotsym(/circle,scale=1.05)


;oplot, wp_submm, pm1, thick=4, linesty=2

oplot, wp_submm, pm, thick=8
oplot, wp_submm, pac, thick=2, color=FSC_Color('brown')
oplot, wp_submm, psi, thick=2, color=FSC_Color('lime green')
;oplot, wp_submm, pdark, thick=2

; vary alignment radii:
; oplot, wp_submm, p10_2, color=FSC_Color('black')
; oplot, wp_submm, p15_2, color=FSC_Color('dark green'), thick=4
; oplot, wp_submm, p10_3, color=FSC_Color('green')



xyouts, 2200, paC(1)  *1.05, 'aC', color=FSC_Color('brown')
xyouts, 2200, psi(1)  *1.05, 'Si', color=FSC_Color('lime green')
;xyouts, 2200, pdark(1)*1.1, 'Dark'

legend, ['Planck XXII', 'Gandilo+16', 'Sharif+18','Ashton+18'], /right, $
        textcolor=[FSC_Color('blue'),FSC_Color('magenta'),FSC_Color('brown'),$
                   FSC_Color('orange')],  charsize = 0.75




xmin =  180.
xmax = 3050.

plot_oi, [wa, wpp], [pga, pp], /xsty,  /ysty, xthick=2,ythick=2,  $
         xtitle='!3 !9l !3(!9m!7m)', charsize=1.3, $
         ytitle = '!7p / p!D850!9m!7m!N', $
         xrange=[xmin, xmax], yrange=[0., 1.4], /nodata, /noerase

xyouts, 200., -0.1,   '200', align=0.5, charsize=1.3
xyouts, 3000., -0.1, '3000', align=0.5, charsize=1.3

if keyword_set(ps) then endps

end

; =======================
pro pl_FIRdef,  ps=ps
;
; Plot deficit of FIR excess not explained by model nu*Fmodel / (nu*Fdata)
; keyword_set  ps then plot to ps file 
; -----------

  print, ' here: pl_FIRdef'
clicht =   2.9979250e+10  
;
; Plot setup:
;
if keyword_set(ps) then begin
  window, 0, xsi=600, ysiz=500
  !p.font=0
  !p.thick=4
  !p.charsize =1.8
  setps, file='pl_xFIR.ps'
endif



restore,/verb, 'ISMsedata.xdr'

wdata       = [w_dh]
Sdata       = [nuf_dh]
errSdata    = [errnuf_dh]


file = 'Output/emis.out'
readcol, file, skipline=2, w, fmod, fac, fSi, fpsi, fDark, fvgr, fvsi, fpah, j_v, /sil

; get wavelengths and scale obserevd nu*F and model to 60mu data [erg/s/sr/H-atom]
i60   = where(w eq 60.)    
i100  = max(where(w ge wdata(0)))    ; & print, w(i100)  , wdata(0)
i140  = max(where(w ge wdata(1))) +1 ; & print, w(i140)  , wdata(1)
i240  = max(where(w ge wdata(2)))    ; & print, w(i240)  , wdata(2)
i350  = max(where(w ge wdata(3)))    ; & print, w(i350)  , wdata(3)
i550  = max(where(w ge wdata(4))) +1 ; & print, w(i550)  , wdata(4)
i850  = max(where(w ge wdata(5)))    ; & print, w(i850)  , wdata(5)
i1400 = max(where(w ge wdata(6))) +1  & print, ' check wave: ',w(i1400) , wdata(6)
i2100 = max(where(w ge wdata(7)))    ; & print, w(i2100) , wdata(7)
i3000 = max(where(w ge wdata(8)))    ; & print, w(i3000) , wdata(8)


; read data and scale to no dark dust model at 100mu
; Compute excess by subtracting no dark dust model from data:
; DIRBE data (Dwek 1996), Planck (Guillet17) and Hensley & Draine 2021,
inorm       = 3 ; this is 100mu of data arrays
fak         = Sdata(inorm)
Sdata       = Sdata/fak
errSdata    = errSdata/fak 
nuf_p       = nuf_p/fak
nuf_dirbe   = nuf_dirbe/fak


fr          = clicht/w*1e4
vfmod       = fmod  * fr
scale_100   = Sdata(inorm)/ vfmod(i100)
vfmod     = fmod * fr * scale_100 > 1.d-20

S0       = 10.^(interpol(alog10(vfmod), w, wdata)) 
S0       = S0/S0(inorm)  ; 100mu model flux =1.

; get 60mu no dark model flux
;S0_60  = vfmod(i60(0))

; convert wavelengths in mm and restrict dirbe data to 100,140, 240mu:
wdata       = wdata   /1000.
w_dirbe     = w_dirbe /1000.
w_dirbe     = w_dirbe(5:7) 
S_dirbe     = nuf_dirbe(5:7) 


; ============== PLotting ========

plot_oi, wdata, S0/Sdata, xr = [0.2, 4], yr =[0.7, 1.2], $
         xthick=4, ythick=4, /xsty, /ysty, $ 
         /nodata, xtitle='!9l (!3mm)', ytitle = '!3 F/F!Dobs!N '

       oplot, wdata, wdata*0.+1, color=FSC_Color('dark gray')
       
       oploterror, wdata, Sdata/Sdata, wdata*0.1, errSdata/S0, /nohat, $
            psym=3, color=FSC_Color('dark gray')

oploterror, w_dirbe, S_dirbe/Sdata, wdata*0.1,S0*0, /nohat, $
            psym=3, color=FSC_Color('dark gray')

oplot, wdata, S0/Sdata, thick=4

oplot, w_dirbe, S_dirbe/Sdata, psym=plotsym(/circle,scale=0.8,/fill),color=FSC_Color('blue')
oplot, w_dirbe, S_dirbe/Sdata, psym=plotsym(/circle,scale=0.8)

oplot, wdata(0:2), Sdata(0:2)/Sdata(0:2), psym=plotsym(/circle,scale=0.8, /fill), color=FSC_Color('blue')
oplot, wdata(3:*), Sdata(3:*)/Sdata(3:*), psym=plotsym(/circle,scale=0.8, /fill), color=FSC_Color('magenta')
oplot, wdata, Sdata/Sdata,  psym=plotsym(/circle,scale=0.8)



oplot, [wdata(0:7), wdata(9:12)], [S0(0:7)/Sdata(0:7), S0(9:12)/Sdata(9:12)],  $
       psym=plotsym(/circle,scale=0.8, /fill)


if keyword_set(ps) then endps

print, ' pl_FIRdef: done'
end


pro pl_sedarkISM, ps=ps, scale350=scale350
  
; ISM pots for emission, reddening, extinction, Srrkowski,
;              and submmPol
; if keyword_set  ps then plot to ps file ps_subpol.ps
; if keyword_set  scale350 and not kerywoprd_set ps then return
;                  afetr scaling factor without plot
; Output:
;  
;  scale350 [g-dust/H-atom] := scale Model [erg/s/sr/g-dust] to Planck 350mu
;              [erg/s/sr/H-atom] to
;
; .r pl_subPol
; .r pl_FIRdef.pro
; .r dirbe_conv.pro
;  
; dirbe_conv, w, flux, ws, dwsm, dwsp, S, xFv, wdirbe, Sdirbe, vF, xvF
; -------- ? 
; Staub zu gas verhaeltnis:
; abuc_tot = 209.5d-6
; abusi    =  29.d-6
; Mdust = (abuc_tot*12. + abusi*168.)*protm
; Mgas  = 1.4*protm
; Umrechnung von pro gramm Staub in pro H-atom
; x_H = (Mdust/Mgas *protm)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


clicht  =  29979250000.0
  
; Hensley & Draine 2021, Table 1 nuInu/Nh [erg/s/sr/H-atom], Guillet
; PLanck and Dwek DIRBE. ; when normalized to 350mu almost identical
; to G17

restore,/verb, 'ISMsedata.xdr'



; ====================================
; -- Dust model:


readcol, './Output/emis.out', skipline=2, w, f, fac, fSi, fpsi, fDark, fvgr, fvsi, fpah, j_v, /sil

dirbe_conv, w, f, w_dirbe, dwm_dirbe, dwp_dirbe, S, xFv, wdirbe, Sdirbe, vFdirbe, scale_dirbe



fr       = clicht/w*1e4
vf       = f  * fr

ii1      = (where(w eq 350.))(0)   ; 350mu
scale350 = nuf_dh(3) / vf(ii1)     ; scale Model to Hensley&Draine 350mu [erg/s/sr/H-atom]
;print, ' wel     _DH - vf   _mod'
;print, w(ii1), nuf_dh(3), vf(ii1)

vf     = f          *fr  * scale350          
vfaC   = fac        *fr  * scale350
vfSi   = fSi        *fr  * scale350
vfdark = fdark      *fr  * scale350
vfvgr  = fvgr       *fr  * scale350
vfvsi  = fvsi       *fr  * scale350
vfpah  = fpah       *fr  * scale350
vfdark = fdark      *fr  * scale350



print, ' scale Model to Planck 350mu [g-dust/H-atom] ', scale350
print, ' scale Model to DIRBE  140mu [g-dust/H-atom] ', scale_dirbe

if keyword_set(scale350) and not keyword_set(ps) then begin
   print, '  -- done: just scale parameter, for plot set keyword /ps '
;   return
endif



; ---------------- Computes underestimate of model to Planck SED

; get wavelengths:
;i60  = max(where(w ge w_dh(3)))     & print, w(i350)   , w_dh(3)
;i100  = max(where(w ge w_dh(3)))     & print, w(i350)   , w_dh(3)
;i140  = max(where(w ge w_dh(3)))     & print, w(i350)   , w_dh(3)
;i240  = max(where(w ge w_dh(3)))     & print, w(i350)   , w_dh(3)
i350  = max(where(w ge w_dh(3)))     & print, w(i350)   , w_dh(3)
i550  = max(where(w ge w_dh(4))) +1   ; & print, w(i550)   , w_dh(4)
i850  = max(where(w ge w_dh(5)))     & print, w(i850)  , w_dh(5)
i1400 = max(where(w ge w_dh(6))) +1   ; & print, w(i1400)  , w_dh(6)
i2100 = max(where(w ge w_dh(7)))      ; & print, w(i2100)  , w_dh(7)
i3000 = max(where(w ge w_dh(8)))      ; & print, w(i3000)  , w_dh(8)


fak = [vf(i350)/ nuf_dh(3), vf(i550)/ nuf_dh(4), vf(i850)/ nuf_dh(5), $
      vf(i1400)/ nuf_dh(6), vf(i2100)/ nuf_dh(7), vf(i3000)/ nuf_dh(8)]


vfmrn = vfac+vfsi
fakmrn = [vfmrn(i350)/ nuf_dh(3), vfmrn(i550)/ nuf_dh(4), $
          vfmrn(i850)/ nuf_dh(5), $
          vfmrn(i1400)/ nuf_dh(6), $
          vfmrn(i2100)/ nuf_dh(7), vfmrn(i3000)/ nuf_dh(8)]



print, ' *** Scaling to Planck   '
print, '                  fmrn   = ', mean(fakmrn)
print, '            fDark+fmrn   = ', mean(fak)


; ================

;
; Plot setup:
;

if keyword_set(ps) then begin
  window, 0, xsi=600, ysiz=500
  !p.font=0
  !p.thick=4
  !p.charsize =1.5
  setps, file='pl_sedarkISM.ps'
endif

!p.thick=1

; -------------
; Plot emission
; -------------

ymin = 2.d-30
plot_oo, w,vF,  xrange=[2.8,3500.],   yrange=[ymin,5d-25],  $
         /xsty,  /ysty, xthick=4,ythick=4,$
         xtickname=[' ', ' ', ' ', ' '], $
         /nodata,thick=6, xtitle='!9l (!9m!3m)', $
         ytitle = '!9n !3I!D!9n!3!N / N!DH!N (erg s!U-1!N sr!U-1!N H!U-1!N)'

ii = where(w gt 2.9 and w le 3500.)

x = [max(w(ii)), w(ii), min(w(ii))]
yp = [ymin,  vf(ii)>ymin, ymin] 
PolyFill, x,yp, Color=FSC_Color('light cyan')



yp = [ymin,  vfdark(ii)>ymin, ymin] 
PolyFill, x,yp, Color=FSC_Color('Charcoal')

oplot, w,  vfpah, linestyle=0, color=FSC_Color('orange'), thick=1
oplot, w,  vfvsi, linestyle=0, color=FSC_Color('forest green'), thick=1
oplot, w,  vfvgr, linestyle=0, color=FSC_Color('sky blue'), thick=1
oplot, w,  vfaC, linestyle=0, color=FSC_Color('brown'), thick=1
oplot, w,  vfSi, linestyle=0, color=FSC_Color('lime green'), thick=1

oplot, w,  vfdark, thick=6      ;, color=FSC_Color('yellow')

oplot, w,  vf,     thick=8



errplot, wdirbe(0:4),0.8*nuf_dirbe(0:4), 1.2*s(0:4), thick=4, color=FSC_Color('gray')
dwsm = (dwm_dirbe - w_dirbe) + wdirbe
dwsp = (dwp_dirbe - w_dirbe) + wdirbe

for i =0,4 do oplot, [dwsm(i),dwsp(i)], [nuf_dirbe(i), nuf_dirbe(i)], thick=6, $
                     color=FSC_Color('dark gray')

oplot, wdirbe(0:4), nuf_dirbe(0:4),   psym=plotsym(/circle,scale=0.7,/fill, $
                                                   color=FSC_Color('magenta'))
oplot, wdirbe(0:4), nuf_dirbe(0:4),   psym=plotsym(/circle,scale=0.7)





oploterror, w_dh,nuf_dh, errnuf_dh, psym=3
oplot, w_dh,nuf_dh, psym=plotsym(/circle,scale=0.8,/fill, color=FSC_Color('magenta'))
oplot, w_dh,nuf_dh, psym=plotsym(/circle,scale=0.8, color=FSC_Color('black'))

oplot, w,  vf,     thick=1



xyouts, 12,  1d-27, 'PAH', charsize= 0.9, color= fsc_color('orange')
xyouts, 20,  1d-29, 'aC' , charsize= 0.9, color= fsc_color('brown')
xyouts, 40,  1d-29, 'Si' , charsize= 0.9, color= fsc_color('lime green')
xyouts, 5,  3d-28, 'vSi', charsize= 0.9, color= fsc_color('forest green'), align=0.5
xyouts, 5,  1.1d-27, 'vGr', charsize= 0.9, color= fsc_color('sky blue'), align=0.5

xyouts, 180, 2d-28, 'Dark' , charsize= 1.2, color= fsc_color('light gray')
xyouts, 180, 1d-28, 'Dust' , charsize= 1.2, color= fsc_color('light gray')

legend, [' ISM '], box=0, charsize=2.0, /right
; linestyle=[0,2], color=[FSC_Color('black'),FSC_Color('magenta')], pos=[30,0.3]


ymin = 2.d-30


 wdata    = [wdirbe(0:4),w_dh]
 Sdata    = [nuf_dirbe(0:4), nuf_dh]
 errSdata = [0.2*nuf_dirbe(0:4), errnuf_dh]


plot_oo, w,vF,  xrange=[2.8,3500.],   yrange=[ymin,5d-25],  $
         /xsty,  /ysty, xthick=4,ythick=4, /noerase, $
         /nodata,thick=6, xtitle='!9l (!9m!3m)', $
         ytitle = '!9n !3I!D!9n!3!N / N!DH!N (erg s!U-1!N sr!U-1!N H!U-1!N)'

print, ' ISM emission plot DONE '

; -----------------------
; Plot deficit of FIR excess not explained by model nu*Fmodel / (nu*Fdata)
; -----------------------

pl_FIRdef


; -----------------------
; read reddening files
; ----------------------

readcol, './Data/FM19_meanExt.redd', x19, redd19, /sil

readcol, './Data/HD046202_G.redd', wdredd, reddHD, rms_upredd, rms_downredd, /sil

readcol, './Data/AvgSerkISM.redd', skipl=2, wfm, efm, rpfm,rmfm, /sil
xfm = 1./wfm


; Kappa.out
rdfloat, './Output/Kappa.out', skipline=3, w, sac, ssac, sasi, $
         sssi, savgr, ssvgr, savsi, ssvsi, sapah, sad, ssd, st, /sil
w   = w*1e4
iv  = (where(w le 0.5477))(0)
ib  = (where(w le 0.45))(0)
;print, w(iv), w(ib)

Rv_obs = 3.1
Rv     = st(iv) / (st(ib) - st(iv))

print,' Rv:   obs, model = ', Rv_obs, Rv

redd_mod = (st/st(iv)   -1.) * Rv


;
; ---------------------
; Plot Reddening : 1/w
; ----------------------

plot, 1./w, redd_mod, xrange=[-0.1,10.99],yrange=[-4.99,16.4], /xsty, /ysty, $
   xthick=4, ythick=4, xtitle='1/!9l!3 (!9m!3m!U-1!N)', $
   ytitle= '!3E(!9l!3-V) / E(B-V) (mag)', /nodata



oplot, xfm, efm, color=fsc_color('dark gray')


oplot, 1./w, ((sac+ssac)/st(iv)  -1.)   * Rv, color=fsc_color('brown')
oplot, 1./w, ((sasi+sssi)/st(iv) -1.)   * Rv, color=fsc_color('lime green')
oplot, 1./w, ((savgr+ssvgr)/st(iv) -1.) * Rv, color=fsc_color('sky blue')
oplot, 1./w, ((savsi+ssvsi)/st(iv) -1.) * Rv, color=fsc_color('forest green')
oplot, 1./w, (sapah/st(iv) -1.)         * Rv, color=fsc_color('orange')

oplot, 1./w, ((sac+ssac+sasi+sssi+savgr+ ssvgr+ savsi+ ssvsi+ sapah+sad+ ssd)/st(iv) -1.) * Rv, color=fsc_color('orange')

oplot, 1./w, ((sad+ssd)/st(iv) -1.)     * Rv, thick=6,  color= fsc_color('Charcoal')

ii = where(xfm gt 8.3)

oploterror, xfm, efm, xfm*0, 0.1*abs(efm) > 0.05, /nohat, $
            color=fsc_color('dark gray'), psym=3

oploterror, xfm(ii), efm(ii), xfm(ii)*0, 0.1*abs(efm(ii)) > 0.05, /nohat, $
            color=fsc_color('gray'), psym=3

oplot, xfm, efm,color=fsc_color('white'), psym=plotsym(/circle,/fill, scale=0.4)

oplot, xfm, efm, psym=plotsym(/circle, scale=0.3),color=fsc_color('dark gray')



oplot, xfm, efm, psym=plotsym(/circle, scale=0.3),color=fsc_color('dark gray')

oplot, xfm(ii), efm(ii), psym=plotsym(/circle, scale=0.3),color=fsc_color('gray')


oplot, 1./w, redd_mod,  color=fsc_color('black'), thick=8

oplot, x19, redd19, color=fsc_color('orange')


xyouts, 6,  0.5, 'Si' , charsize= 0.9, color= fsc_color('lime green')
xyouts, 3,  0.5, 'aC' , charsize= 0.9, color= fsc_color('brown')
xyouts, 10, 3, 'vSi', charsize= 0.9, color= fsc_color('forest green')
xyouts, 10,  0.5, 'vGr', charsize= 0.9, color= fsc_color('sky blue')
xyouts, 10,  -1.8, 'PAH', charsize= 0.9, color= fsc_color('orange')
xyouts, 10,  -4.4, 'Dark Dust', charsize= 1.3, align=1, color= fsc_color('Charcoal')



oploterror, 1./wdredd(0:6), reddHD(0:6), wdredd(0:6)*0, 0.1*reddHD(0:6), $
            color=fsc_color('gray'), psym=3

oplot, 1./wdredd(0:6), reddHD(0:6), psym=plotsym(/circle,/fill, scale=0.7), $
       color=fsc_color('gray')
oplot, 1./wdredd(0:6), reddHD(0:6), psym=plotsym(/circle, scale=0.6)


legend, ['Model', 'FM19', 'FM07 (R!DV!N=3.1)', 'HD046202'], charsize= 0.9, $
        textcolor=[FSC_Color('black'),FSC_Color('magenta'), FSC_Color('dark gray'), FSC_Color('gray')]



print, 'reddening plot done '


;
; ========= Prepare plotting MIR extinction
; model

svsg = savgr + ssvgr + savsi + ssvsi 

; FM extinction
tauHD     = reddHD  /Rv_obs+1.
tauFM     = efm     /abs(efm(0)) +1.

tau19     = redd19/Rv_obs+1.


; NIR extinction Xiaodian Chen et al 2018 ApJ 859 137, Table3
; schale Chen to FM extinction at 2.2mu 
 
readcol, './Data/Chen19nirExt.tab', /sil, skipl=4, wchen, $
         Achen1, errAchen1, Achen2, errAchen2
tchen1    = Achen1   /1.0857
tchen2    = Achen2   /1.0857
errtchen1 = sqrt((errAchen1/1.0857)^2 + (0.001 * tchen1)^2)
errtchen2 = sqrt((errAchen2/1.0857)^2 + (0.001 * tchen2)^2)


wnorm     = 1.633
ii        = max((where(1./xfm ge wnorm)))
iic       = (where(wchen eq wnorm))(0)

tchen1    = tchen1    * tauFM(ii)/tchen1(iic)
tchen2    = tchen2    * tauFM(ii)/tchen2(iic)
errtchen1 = errtchen1 * tauFM(ii)/tchen1(iic)
errtchen2 = errtchen2 * tauFM(ii)/tchen2(iic)

; Indebetouw+05
tIb = tchen1(iic)*(wchen/wchen(iic))^(-1.)



; =============================
; plotting: opt-MIR Extinction
; =============================

xmin = 0.5
xmax = 7.
ymin = 1.2e-2
ymax = 1.2

plot_oo, w, st/st(iv), xrange=[xmin,xmax],yrange=[ymin,ymax], $
         /xsty, /ysty, xthick=4, ythick=4, charsize=1.3, $
         xtitle='!9l!3 (!9m!3m!N)', ytitle= '!9t!3 / !9t!3!DV!N', $
         xtickname=[' ', ' ', ' ', ' ',' ', ' '], $
         /nodata

oplot,  1./xfm, tauFM, color=fsc_color('magenta'), thick=6

ii0 = min(where(w le xmin))
ii1 = min(where(w le xmax))

 y   = (sad+ssd)/st(iv)
xpol = [xmax,   w(ii1:ii0), xmin]
ypol = [y(ii1), y(ii1:ii0), y(ii1)] > ymin

;PolyFill, xpol,ypol, Color=FSC_Color('charcoal')

PolyFill, xpol,ypol, Color=FSC_Color('gray')






oplot, w, st/st(iv), color=fsc_color('black'), thick=8
oplot, w, (sasi+sssi)/st(iv), color=fsc_color('lime green'), thick=2, linesty=2
oplot, w, (sac+ssac)/st(iv), color=fsc_color('brown'), thick=2, linesty=2
;oplot, w, (sad+ssd)/st(iv),  color=fsc_color('white'), thick=1, linesty=0
;oplot, w, (sad+ssd)/st(iv),  color=fsc_color('slate gray'), thick=1, linesty=0


oploterror,  wdredd, tauHD, wdredd*0, 0.1*tauHD, $
             psym=plotsym(/box,scale=0.7,/fill), $
             color=FSC_Color('salmon')
oplot,       wdredd, tauHD, psym=plotsym(/box,scale=0.7)


oploterror,  wchen, tchen1, 0*wchen, tchen1*0.1, $
    psym=plotsym(/circle,scale=0.6,/fill), color=fsc_color('blue')
oplot,  wchen, tchen1, psym=plotsym(/circle,scale=0.6)

oploterror,  wchen, tchen2, 0*wchen, tchen2*0.1, $
    psym=plotsym(/circle,scale=0.6,/fill), color=fsc_color('blue')
oplot,  wchen, tchen2, psym=plotsym(/circle,scale=0.6)

oploterror,  wchen, tib, 0*wchen, tib*0.15, $
             psym=plotsym(/circle,scale=0.6,/fill), $
             color=fsc_color('forest green')
oplot,  wchen, tib,  psym=plotsym(/circle,scale=0.6)

oplot,  wchen, tib, psym=plotsym(/circle,scale=0.6)

oplot,  1./xfm, tauFM, color=fsc_color('magenta'), thick=2


legend, ['Model', 'aC', 'Si', 'Dark Dust'], linesty=[0,2,2,0], /bottom, charsize= 0.9, $
        color=[FSC_Color('black'),FSC_Color('brown'),FSC_Color('lime green'), FSC_Color('slate gray')], $
        textcolor=[FSC_Color('black'),FSC_Color('brown'), FSC_Color('lime green'), FSC_Color('white')], box=0, thick=2





legend, pos=[3., 0.9], ['FM07 (R!DV!N=3.1)', 'HD046202', $
         'Chen+18', 'Indebetouw+05'], charsize= 0.9, $
      color=[FSC_Color('magenta'),  $
     FSC_Color('dark gray'),FSC_Color('blue'), FSC_Color('forest green')], $
  textcolor=[FSC_Color('magenta'),  $
     FSC_Color('salmon'),FSC_Color('blue'), FSC_Color('forest green')], box=0


oplot,  1./xfm, tauFM, color=fsc_color('magenta'), thick=2





plot_oo, w, st/st(iv), xrange=[xmin,xmax],yrange=[ymin,ymax], $
         /xsty, /ysty, xthick=4, ythick=4, charsize=1.3, $
         xtitle='!9l!3 (!9m!3m!N)', ytitle= '!9t!3 / !9t!3!DV!N', $
         /noerase, /nodata

xyouts, xmin, 0.0088, '0.5', align=0.5, charsize=1.3 
xyouts, xmax, 0.0088, '7',   align=0.5, charsize=1.3

print, ' ISM reddening plot in pl_reddavg.ps'


;
; -------------
; Serkowski  ISM = HD046202:
; -------------
;
fpol = './Data/AvgSerkISM.pol'
readcol, fpol,  wpd, Int, dInt, Qint, dQint, nQ, Uint, dUint, nU, $
          Pdata, rmspdata,theta, dtheta, skip=0, /sil

np      = n_elements(wpd)
kp      = where(max(pdata[2:np-3]) eq pdata and $
                wpd ge wpd(2) and wpd le wpd(np-3))
wpmax= wpd(kp(0))
if wpmax le 0.4 or wpmax ge 0.9 then stop, ' wpmax cannot be corect units?'

; Polarisation dust model : will be normalised to pmax, wpm (cm)
rdfloat, './Output/PolKappa.out', skipline=3,  wpm,spac, spsi,spd,st, /sil

wpm = wpm *1e4
ii  = where(wpm ge (min(wpd)-1e-4) and wpm le (max(wpd)+1e-4), npm)
if npm ne np then print, ' WARNING  check number of pol data in wpm, wpd grid', npm, np

 
kk = max(where(wpm ge wpmax))
dw0 = abs(wpm(kk+1) - wpmax)
dw1 = abs(wpm(kk)   - wpmax)
if dw0 lt dw1 then kk = kk+1


wpm = wpm(ii)

spac  = spac(ii) 
spsi  = spsi(ii) 
spd   = spd(ii)
pm    = spac + spsi + spd

pm  = pm * max(Pdata(3:np-3))
fak = max(pm)
pm  = pm/fak


iimax = where(pm ge 0.99999)
print, ' *** wavelength of max polarisation '
print, '                   w(data)  = ', wpmax
print, '                   w(model) = ', wpm(iimax)



; ==============
; Polarisation
; ==============


plot_oi, wpd, pdata, psym = 10, xr=[min(wpd)*0.9, max(wpd)*1.1], $
         xthick=4, ythick=4,  charsize=1.3, $
      /xsty, /ysty, /nodata, yr=[0, max(pdata)*1.1], $
       xtitle='!9l !3(!9m!3m)', ytitle = 'p / p!Dmax!N'


oploterror, wpd, pdata, 0*wpd, rmspdata, psym=3, color=fsc_color('gray')

oplot, wpm, pm * max(Pdata(3:np-3)), color=fsc_color('black'), thick=8


oplot, wpd, pdata, psym=plotsym(/circle,scale=0.6,/fill), color=fsc_color('magenta')
oplot, wpd, pdata, psym=plotsym(/circle,scale=0.6)


oplot, wpm, spsi/fak, color=fsc_color('lime green') 
oplot, wpm, spac/fak, color=fsc_color('brown') 

if max(spd/fak) gt 0.01 then $
oplot, wpm, spd/fak, color=fsc_color('charcoal'), thick=6



xyouts, 0.2, -0.09, '0.2', align=0.5, charsize=1.3
xyouts, 4,   -0.09, '4', align=0.5, charsize=1.3


legend, ['Model', 'aC', 'Si'], linesty=[0,0,0], /right, $
        charsize= 0.9, $
     color=[FSC_Color('black'),FSC_Color('brown'),FSC_Color('lime green')], $
 textcolor=[FSC_Color('black'),FSC_Color('brown'),FSC_Color('lime green')]

; in case with dark dust:
;
;legend, ['Model', 'aC', 'Si', 'Dark Dust'], linesty=[0,2,2,0], /right, $
;        charsize= 0.9, box=0, $
;        color=[FSC_Color('black'),FSC_Color('brown'),FSC_Color('lime green'), $
;         FSC_Color('Charcoal')], $
;        textcolor=[FSC_Color('black'),FSC_Color('brown'), $
;                   FSC_Color('lime green'), FSC_Color('white')]
;


print, ' ISM Serkowski plot done: pl_sedarkISM.ps ' 





; =======================
;    submm Polarisation 
; =======================
;


;print, ' no     submm Polarisation  needs Output/emipol.out'
          pl_subpol



; ================================
; plotting: cross section
;

xmin = 0.09
xmax = 2000.
ymin = 1e-5
ymax = 7

plot_oo, w, st/st(iv), xrange=[xmin,xmax],yrange=[ymin,ymax], $
         /xsty, /ysty, xthick=4, ythick=4, charsize=1.3, $
         xtickname=[' ', ' ', ' ', ' ',' ', ' '], $
         xtitle='!9l!3 (!9m!3m!N)', ytitle= '!9K!3 / !9K!3!DV!N', $
         /nodata 

oplot, w, sapah/st(iv), color=fsc_color('orange')
oplot, w, svsg/st(iv), color=fsc_color('sky blue')
oplot, w, (sasi+sssi)/st(iv), color=fsc_color('forest green')
oplot, w, (sac+ssac)/st(iv), color=fsc_color('brown')
oplot, w, (sad+ssd)/st(iv),  color=fsc_color('charcoal'), thick=6
oplot, w, st/st(iv), thick=8

legend, ['total', 'Dark Dust', 'aC', 'Si', 'VSG', 'PAH'], $
linesty=[0,0,0,0,0,0], /right, charsize= 0.9, $
color=[FSC_Color('black'), FSC_Color('Charcoal'), FSC_Color('brown'), $
FSC_Color('forest green'), FSC_Color('sky blue'), FSC_Color('orange'), $
FSC_Color('orange')], $
textcolor=[FSC_Color('black'), FSC_Color('Charcoal'),$
FSC_Color('brown'), FSC_Color('forest green'), $
            FSC_Color('sky blue'),FSC_Color('orange'), FSC_Color('orange')]


plot_oo, w, st/st(iv), xrange=[xmin,xmax],yrange=[ymin,ymax], $
         /xsty, /ysty, xthick=4, ythick=4, charsize=1.3, $
         xtitle='!9l!3 (!9m!3m!N)', ytitle= '!9K!3 / !9K!3!DV!N', $
         xtickname=['0.1', '1','10','100','1000'], $
         /noerase, /nodata


if keyword_set(ps) then endps


print
com = 'ps2pdf pl_sedarkISM.ps pl_sedarkISM.pdf
spawn, com
print, com
print
print, ' all plots in file:  --->  pl_sedarkISM.pdf  :  DONE'
print

;
; --------- done -------
;




end
